
use Test::More  'no_plan';
BEGIN { use_ok('Pg::Pcurse') };
#BEGIN { use_ok('Pg::Pcurse::Widget') };
BEGIN { use_ok('Pg::Pcurse::Misc') };
BEGIN { use_ok('Pg::Pcurse::Query0') };
BEGIN { use_ok('Pg::Pcurse::Query1') };
BEGIN { use_ok('Pg::Pcurse::Query2') };
BEGIN { use_ok('Pg::Pcurse::Query3') };

